@@ -1,18 +1,34 @@
 """Utilities for type argument inference."""
 
-from typing import List, Optional, Sequence
+from typing import List, Optional, Sequence, NamedTuple
 
 from mypy.constraints import (
     infer_constraints, infer_constraints_for_callable, SUBTYPE_OF, SUPERTYPE_OF
 )
-from mypy.types import Type, TypeVarId, CallableType
+from mypy.types import Type, TypeVarId, CallableType, Instance
+from mypy.nodes import ArgKind
 from mypy.solve import solve_constraints
 
 
+class ArgumentInferContext(NamedTuple):
+    """Type argument inference context.
+
+    We need this because we pass around ``Mapping`` and ``Iterable`` types.
+    These types are only known by ``TypeChecker`` itself.
+    It is required for ``*`` and ``**`` argument inference.
+
+    https://github.com/python/mypy/issues/11144
+    """
+
+    mapping_type: Instance
+    iterable_type: Instance
+
+
 def infer_function_type_arguments(callee_type: CallableType,
                                   arg_types: Sequence[Optional[Type]],
-                                  arg_kinds: List[int],
+                                  arg_kinds: List[ArgKind],
                                   formal_to_actual: List[List[int]],
+                                  context: ArgumentInferContext,
                                   strict: bool = True) -> List[Optional[Type]]:
     """Infer the type arguments of a generic function.
 
@@ -29,7 +45,7 @@ def infer_function_type_arguments(callee_type: CallableType,
     """
     # Infer constraints.
     constraints = infer_constraints_for_callable(
-        callee_type, arg_types, arg_kinds, formal_to_actual)
+        callee_type, arg_types, arg_kinds, formal_to_actual, context)
 
     # Solve constraints.
     type_vars = callee_type.type_var_ids()
