@@ -28,8 +28,8 @@ from mypy.split_namespace import SplitNamespace
 
 from mypy.version import __version__
 
-orig_stat = os.stat  # type: Final
-MEM_PROFILE = False  # type: Final  # If True, dump memory profile
+orig_stat: Final = os.stat
+MEM_PROFILE: Final = False  # If True, dump memory profile
 
 
 def stat_proxy(path: str) -> os.stat_result:
@@ -48,13 +48,16 @@ def main(script_path: Optional[str],
          stdout: TextIO,
          stderr: TextIO,
          args: Optional[List[str]] = None,
+         clean_exit: bool = False,
          ) -> None:
     """Main entry point to the type checker.
 
     Args:
         script_path: Path to the 'mypy' script (used for finding data files).
         args: Custom command-line arguments.  If not given, sys.argv[1:] will
-        be used.
+            be used.
+        clean_exit: Don't hard kill the process on exit. This allows catching
+            SystemExit.
     """
     util.check_python_version('mypy')
     t0 = time.time()
@@ -66,6 +69,8 @@ def main(script_path: Optional[str],
     fscache = FileSystemCache()
     sources, options = process_options(args, stdout=stdout, stderr=stderr,
                                        fscache=fscache)
+    if clean_exit:
+        options.fast_exit = False
 
     formatter = util.FancyFormatter(stdout, stderr, options.show_error_codes)
 
@@ -80,8 +85,12 @@ def main(script_path: Optional[str],
         fail("error: --install-types not supported with incremental mode disabled",
              stderr, options)
 
+    if options.install_types and options.python_executable is None:
+        fail("error: --install-types not supported without python executable or site packages",
+             stderr, options)
+
     if options.install_types and not sources:
-        install_types(options.cache_dir, formatter, non_interactive=options.non_interactive)
+        install_types(formatter, options, non_interactive=options.non_interactive)
         return
 
     res, messages, blockers = run_build(sources, options, fscache, t0, stdout, stderr)
@@ -90,7 +99,7 @@ def main(script_path: Optional[str],
         missing_pkgs = read_types_packages_to_install(options.cache_dir, after_run=True)
         if missing_pkgs:
             # Install missing type packages and rerun build.
-            install_types(options.cache_dir, formatter, after_run=True, non_interactive=True)
+            install_types(formatter, options, after_run=True, non_interactive=True)
             fscache.flush()
             print()
             res, messages, blockers = run_build(sources, options, fscache, t0, stdout, stderr)
@@ -117,8 +126,7 @@ def main(script_path: Optional[str],
         stdout.flush()
 
     if options.install_types and not options.non_interactive:
-        result = install_types(options.cache_dir, formatter, after_run=True,
-                               non_interactive=False)
+        result = install_types(formatter, options, after_run=True, non_interactive=False)
         if result:
             print()
             print("note: Run mypy again for up-to-date results with installed types")
@@ -208,11 +216,11 @@ class AugmentedHelpFormatter(argparse.RawDescriptionHelpFormatter):
 
 
 # Define pairs of flag prefixes with inverse meaning.
-flag_prefix_pairs = [
+flag_prefix_pairs: Final = [
     ('allow', 'disallow'),
     ('show', 'hide'),
-]  # type: Final
-flag_prefix_map = {}  # type: Final[Dict[str, str]]
+]
+flag_prefix_map: Final[Dict[str, str]] = {}
 for a, b in flag_prefix_pairs:
     flag_prefix_map[a] = b
     flag_prefix_map[b] = a
@@ -237,8 +245,8 @@ class PythonExecutableInferenceError(Exception):
 def python_executable_prefix(v: str) -> List[str]:
     if sys.platform == 'win32':
         # on Windows, all Python executables are named `python`. To handle this, there
-        # is the `py` launcher, which can be passed a version e.g. `py -3.5`, and it will
-        # execute an installed Python 3.5 interpreter. See also:
+        # is the `py` launcher, which can be passed a version e.g. `py -3.8`, and it will
+        # execute an installed Python 3.8 interpreter. See also:
         # https://docs.python.org/3/using/windows.html#python-launcher-for-windows
         return ['py', '-{}'.format(v)]
     else:
@@ -282,11 +290,11 @@ def infer_python_executable(options: Options,
     options.python_executable = python_executable
 
 
-HEADER = """%(prog)s [-h] [-v] [-V] [more options; see below]
-            [-m MODULE] [-p PACKAGE] [-c PROGRAM_TEXT] [files ...]"""  # type: Final
+HEADER: Final = """%(prog)s [-h] [-v] [-V] [more options; see below]
+            [-m MODULE] [-p PACKAGE] [-c PROGRAM_TEXT] [files ...]"""
 
 
-DESCRIPTION = """
+DESCRIPTION: Final = """
 Mypy is a program that will type check your Python code.
 
 Pass in any files or folders you want to type check. Mypy will
@@ -307,11 +315,11 @@ You can also use a config file to configure mypy instead of using
 command line flags. For more details, see:
 
 - https://mypy.readthedocs.io/en/stable/config_file.html
-"""  # type: Final
+"""
 
-FOOTER = """Environment variables:
+FOOTER: Final = """Environment variables:
   Define MYPYPATH for additional module search path entries.
-  Define MYPY_CACHE_DIR to override configuration cache_dir path."""  # type: Final
+  Define MYPY_CACHE_DIR to override configuration cache_dir path."""
 
 
 class CapturableArgumentParser(argparse.ArgumentParser):
@@ -434,8 +442,8 @@ def process_options(args: List[str],
                                       stdout=stdout,
                                       stderr=stderr)
 
-    strict_flag_names = []  # type: List[str]
-    strict_flag_assignments = []  # type: List[Tuple[str, bool]]
+    strict_flag_names: List[str] = []
+    strict_flag_assignments: List[Tuple[str, bool]] = []
 
     def add_invertible_flag(flag: str,
                             *,
@@ -769,7 +777,7 @@ def process_options(args: List[str],
         dest='shadow_file', action='append',
         help="When encountering SOURCE_FILE, read and type check "
              "the contents of SHADOW_FILE instead.")
-    add_invertible_flag('--fast-exit', default=False, help=argparse.SUPPRESS,
+    add_invertible_flag('--fast-exit', default=True, help=argparse.SUPPRESS,
                         group=internals_group)
 
     report_group = parser.add_argument_group(
@@ -852,8 +860,6 @@ def process_options(args: List[str],
     # Must be followed by another flag or by '--' (and then only file args may follow).
     parser.add_argument('--cache-map', nargs='+', dest='special-opts:cache_map',
                         help=argparse.SUPPRESS)
-    # PEP 612 support is a work in progress, hide it from users
-    parser.add_argument('--wip-pep-612', action="store_true", help=argparse.SUPPRESS)
 
     # options specifying code to check
     code_group = parser.add_argument_group(
@@ -866,11 +872,13 @@ def process_options(args: List[str],
         group=code_group)
     code_group.add_argument(
         "--exclude",
+        action="append",
         metavar="PATTERN",
-        default="",
+        default=[],
         help=(
             "Regular expression to match file names, directory names or paths which mypy should "
-            "ignore while recursively discovering files to check, e.g. --exclude '/setup\\.py$'"
+            "ignore while recursively discovering files to check, e.g. --exclude '/setup\\.py$'. "
+            "May be specified more than once, eg. --exclude a --exclude b"
         )
     )
     code_group.add_argument(
@@ -1009,6 +1017,11 @@ def process_options(args: List[str],
 
         process_cache_map(parser, special_opts, options)
 
+    # An explicitly specified cache_fine_grained implies local_partial_types
+    # (because otherwise the cache is not compatible with dmypy)
+    if options.cache_fine_grained:
+        options.local_partial_types = True
+
     # Let logical_deps imply cache_fine_grained (otherwise the former is useless).
     if options.logical_deps:
         options.cache_fine_grained = True
@@ -1141,20 +1154,21 @@ def read_types_packages_to_install(cache_dir: str, after_run: bool) -> List[str]
         return [line.strip() for line in f.readlines()]
 
 
-def install_types(cache_dir: str,
-                  formatter: util.FancyFormatter,
+def install_types(formatter: util.FancyFormatter,
+                  options: Options,
                   *,
                   after_run: bool = False,
                   non_interactive: bool = False) -> bool:
     """Install stub packages using pip if some missing stubs were detected."""
-    packages = read_types_packages_to_install(cache_dir, after_run)
+    packages = read_types_packages_to_install(options.cache_dir, after_run)
     if not packages:
         # If there are no missing stubs, generate no output.
         return False
     if after_run and not non_interactive:
         print()
     print('Installing missing stub packages:')
-    cmd = [sys.executable, '-m', 'pip', 'install'] + packages
+    assert options.python_executable, 'Python executable required to install types'
+    cmd = [options.python_executable, '-m', 'pip', 'install'] + packages
     print(formatter.style(' '.join(cmd), 'none', bold=True))
     print()
     if not non_interactive:
