@@ -47,7 +47,7 @@ class MetadataStore:
         If mtime is specified, set it as the mtime of the entry. Otherwise,
         the current time is used.
 
-        Returns True if the entry is succesfully written, False otherwise.
+        Returns True if the entry is successfully written, False otherwise.
         """
 
     @abstractmethod
@@ -146,8 +146,7 @@ CREATE TABLE IF NOT EXISTS files (
 CREATE INDEX IF NOT EXISTS path_idx on files(path);
 '''
 # No migrations yet
-MIGRATIONS = [
-]  # type: List[str]
+MIGRATIONS: List[str] = []
 
 
 def connect_db(db_file: str) -> 'sqlite3.Connection':
