@@ -4,6 +4,7 @@ This is conceptually part of mypy.semanal (semantic analyzer pass 2).
 """
 
 from typing import List, Tuple, Optional, Union, cast
+from typing_extensions import Final
 
 from mypy.nodes import (
     Expression, Context, TypeInfo, AssignmentStmt, NameExpr, CallExpr, RefExpr, StrExpr,
@@ -12,6 +13,18 @@ from mypy.nodes import (
 )
 from mypy.semanal_shared import SemanticAnalyzerInterface
 from mypy.options import Options
+from mypy.types import get_proper_type, LiteralType, ENUM_REMOVED_PROPS
+
+# Note: 'enum.EnumMeta' is deliberately excluded from this list. Classes that directly use
+# enum.EnumMeta do not necessarily automatically have the 'name' and 'value' attributes.
+ENUM_BASES: Final = frozenset((
+    'enum.Enum', 'enum.IntEnum', 'enum.Flag', 'enum.IntFlag', 'enum.StrEnum',
+))
+ENUM_SPECIAL_PROPS: Final = frozenset((
+    'name', 'value', '_name_', '_value_', *ENUM_REMOVED_PROPS,
+    # Also attributes from `object`:
+    '__module__', '__annotations__', '__doc__', '__slots__', '__dict__',
+))
 
 
 class EnumCallAnalyzer:
@@ -62,7 +75,7 @@ class EnumCallAnalyzer:
         if not isinstance(callee, RefExpr):
             return None
         fullname = callee.fullname
-        if fullname not in ('enum.Enum', 'enum.IntEnum', 'enum.Flag', 'enum.IntFlag'):
+        if fullname not in ENUM_BASES:
             return None
         items, values, ok = self.parse_enum_call_args(call, fullname.split('.')[-1])
         if not ok:
@@ -161,6 +174,23 @@ class EnumCallAnalyzer:
                         "%s() with dict literal requires string literals" % class_name, call)
                 items.append(key.value)
                 values.append(value)
+        elif isinstance(args[1], RefExpr) and isinstance(args[1].node, Var):
+            proper_type = get_proper_type(args[1].node.type)
+            if (proper_type is not None
+                    and isinstance(proper_type, LiteralType)
+                    and isinstance(proper_type.value, str)):
+                fields = proper_type.value
+                for field in fields.replace(',', ' ').split():
+                    items.append(field)
+            elif args[1].node.is_final and isinstance(args[1].node.final_value, str):
+                fields = args[1].node.final_value
+                for field in fields.replace(',', ' ').split():
+                    items.append(field)
+            else:
+                return self.fail_enum_call_arg(
+                    "%s() expects a string, tuple, list or dict literal as the second argument" %
+                    class_name,
+                    call)
         else:
             # TODO: Allow dict(x=1, y=2) as a substitute for {'x': 1, 'y': 2}?
             return self.fail_enum_call_arg(
