@@ -57,9 +57,9 @@ from mypy.nodes import (
 from mypy.traverser import TraverserVisitor
 from mypy.types import (
     Type, SyntheticTypeVisitor, Instance, AnyType, NoneType, CallableType, ErasedType, DeletedType,
-    TupleType, TypeType, TypeVarType, TypedDictType, UnboundType, UninhabitedType, UnionType,
+    TupleType, TypeType, TypedDictType, UnboundType, UninhabitedType, UnionType,
     Overloaded, TypeVarType, TypeList, CallableArgument, EllipsisType, StarType, LiteralType,
-    RawExpressionType, PartialType, PlaceholderType, TypeAliasType
+    RawExpressionType, PartialType, PlaceholderType, TypeAliasType, ParamSpecType
 )
 from mypy.util import get_prefix, replace_object_state
 from mypy.typestate import TypeState
@@ -173,7 +173,8 @@ class NodeReplaceVisitor(TraverserVisitor):
         node.defs.body = self.replace_statements(node.defs.body)
         info = node.info
         for tv in node.type_vars:
-            self.process_type_var_def(tv)
+            if isinstance(tv, TypeVarType):
+                self.process_type_var_def(tv)
         if info:
             if info.is_named_tuple:
                 self.process_synthetic_type_info(info)
@@ -407,6 +408,9 @@ class TypeReplaceVisitor(SyntheticTypeVisitor[None]):
         for value in typ.values:
             value.accept(self)
 
+    def visit_param_spec(self, typ: ParamSpecType) -> None:
+        pass
+
     def visit_typeddict_type(self, typ: TypedDictType) -> None:
         for value_type in typ.items.values():
             value_type.accept(self)
