@@ -1,9 +1,11 @@
 """Test cases for graph processing code in build.py."""
 
+import sys
 from typing import AbstractSet, Dict, Set, List
 
 from mypy.test.helpers import assert_equal, Suite
 from mypy.build import BuildManager, State, BuildSourceSet
+from mypy.modulefinder import SearchPaths
 from mypy.build import topsort, strongly_connected_components, sorted_components, order_ascc
 from mypy.version import __version__
 from mypy.options import Options
@@ -20,16 +22,13 @@ class GraphSuite(Suite):
         b = frozenset({'B'})
         c = frozenset({'C'})
         d = frozenset({'D'})
-        data = {a: {b, c}, b: {d}, c: {d}}  # type: Dict[AbstractSet[str], Set[AbstractSet[str]]]
+        data: Dict[AbstractSet[str], Set[AbstractSet[str]]] = {a: {b, c}, b: {d}, c: {d}}
         res = list(topsort(data))
         assert_equal(res, [{d}, {b, c}, {a}])
 
     def test_scc(self) -> None:
-        vertices = {'A', 'B', 'C', 'D'}
-        edges = {'A': ['B', 'C'],
-                 'B': ['C'],
-                 'C': ['B', 'D'],
-                 'D': []}  # type: Dict[str, List[str]]
+        vertices = {"A", "B", "C", "D"}
+        edges: Dict[str, List[str]] = {"A": ["B", "C"], "B": ["C"], "C": ["B", "D"], "D": []}
         sccs = set(frozenset(x) for x in strongly_connected_components(vertices, edges))
         assert_equal(sccs,
                      {frozenset({'A'}),
@@ -39,19 +38,23 @@ class GraphSuite(Suite):
     def _make_manager(self) -> BuildManager:
         errors = Errors()
         options = Options()
-        fscache = FileSystemCache(options.python_version)
+        fscache = FileSystemCache()
+        search_paths = SearchPaths((), (), (), ())
         manager = BuildManager(
             data_dir='',
-            lib_path=[],
+            search_paths=search_paths,
             ignore_prefix='',
             source_set=BuildSourceSet([]),
             reports=Reports('', {}),
             options=options,
             version_id=__version__,
             plugin=Plugin(options),
+            plugins_snapshot={},
             errors=errors,
             flush_errors=lambda msgs, serious: None,
             fscache=fscache,
+            stdout=sys.stdout,
+            stderr=sys.stderr,
         )
         return manager
 
