@@ -1,11 +1,12 @@
 """Utilities for emitting C code."""
 
-from mypy.ordered_dict import OrderedDict
-from typing import List, Set, Dict, Optional, Callable, Union
+from mypy.backports import OrderedDict
+from typing import List, Set, Dict, Optional, Callable, Union, Tuple
+import sys
 
 from mypyc.common import (
     REG_PREFIX, ATTR_PREFIX, STATIC_PREFIX, TYPE_PREFIX, NATIVE_PREFIX,
-    FAST_ISINSTANCE_MAX_SUBCLASSES,
+    FAST_ISINSTANCE_MAX_SUBCLASSES, use_vectorcall
 )
 from mypyc.ir.ops import BasicBlock, Value
 from mypyc.ir.rtypes import (
@@ -14,12 +15,13 @@ from mypyc.ir.rtypes import (
     is_list_rprimitive, is_dict_rprimitive, is_set_rprimitive, is_tuple_rprimitive,
     is_none_rprimitive, is_object_rprimitive, object_rprimitive, is_str_rprimitive,
     int_rprimitive, is_optional_type, optional_value_type, is_int32_rprimitive,
-    is_int64_rprimitive, is_bit_rprimitive
+    is_int64_rprimitive, is_bit_rprimitive, is_range_rprimitive, is_bytes_rprimitive
 )
 from mypyc.ir.func_ir import FuncDecl
 from mypyc.ir.class_ir import ClassIR, all_concrete_classes
 from mypyc.namegen import NameGenerator, exported_name
 from mypyc.sametype import is_same_type
+from mypyc.codegen.literals import Literals
 
 
 class HeaderDeclaration:
@@ -73,7 +75,7 @@ class EmitterContext:
         self.group_name = group_name
         self.group_map = group_map or {}
         # Groups that this group depends on
-        self.group_deps = set()  # type: Set[str]
+        self.group_deps: Set[str] = set()
 
         # The map below is used for generating declarations and
         # definitions at the top of the C file. The main idea is that they can
@@ -82,7 +84,31 @@ class EmitterContext:
         # A map of a C identifier to whatever the C identifier declares. Currently this is
         # used for declaring structs and the key corresponds to the name of the struct.
         # The declaration contains the body of the struct.
-        self.declarations = OrderedDict()  # type: Dict[str, HeaderDeclaration]
+        self.declarations: Dict[str, HeaderDeclaration] = OrderedDict()
+
+        self.literals = Literals()
+
+
+class ErrorHandler:
+    """Describes handling errors in unbox/cast operations."""
+
+
+class AssignHandler(ErrorHandler):
+    """Assign an error value on error."""
+
+
+class GotoHandler(ErrorHandler):
+    """Goto label on error."""
+
+    def __init__(self, label: str) -> None:
+        self.label = label
+
+
+class ReturnHandler(ErrorHandler):
+    """Return a constant value on error."""
+
+    def __init__(self, value: str) -> None:
+        self.value = value
 
 
 class Emitter:
@@ -90,11 +116,14 @@ class Emitter:
 
     def __init__(self,
                  context: EmitterContext,
-                 value_names: Optional[Dict[Value, str]] = None) -> None:
+                 value_names: Optional[Dict[Value, str]] = None,
+                 capi_version: Optional[Tuple[int, int]] = None,
+                 ) -> None:
         self.context = context
+        self.capi_version = capi_version or sys.version_info[:2]
         self.names = context.names
         self.value_names = value_names or {}
-        self.fragments = []  # type: List[str]
+        self.fragments: List[str] = []
         self._indent = 0
 
     # Low-level operations
@@ -251,6 +280,9 @@ class Emitter:
 
         return result
 
+    def use_vectorcall(self) -> bool:
+        return use_vectorcall(self.capi_version)
+
     def emit_undefined_attr_check(self, rtype: RType, attr_expr: str,
                                   compare: str,
                                   unlikely: bool = False) -> None:
@@ -316,35 +348,56 @@ class Emitter:
                 is_type=True,
             )
 
-    def emit_inc_ref(self, dest: str, rtype: RType) -> None:
+    def emit_inc_ref(self, dest: str, rtype: RType, *, rare: bool = False) -> None:
         """Increment reference count of C expression `dest`.
 
         For composite unboxed structures (e.g. tuples) recursively
         increment reference counts for each component.
+
+        If rare is True, optimize for code size and compilation speed.
         """
         if is_int_rprimitive(rtype):
-            self.emit_line('CPyTagged_IncRef(%s);' % dest)
+            if rare:
+                self.emit_line('CPyTagged_IncRef(%s);' % dest)
+            else:
+                self.emit_line('CPyTagged_INCREF(%s);' % dest)
         elif isinstance(rtype, RTuple):
             for i, item_type in enumerate(rtype.types):
                 self.emit_inc_ref('{}.f{}'.format(dest, i), item_type)
         elif not rtype.is_unboxed:
+            # Always inline, since this is a simple op
             self.emit_line('CPy_INCREF(%s);' % dest)
         # Otherwise assume it's an unboxed, pointerless value and do nothing.
 
-    def emit_dec_ref(self, dest: str, rtype: RType, is_xdec: bool = False) -> None:
+    def emit_dec_ref(self,
+                     dest: str,
+                     rtype: RType,
+                     *,
+                     is_xdec: bool = False,
+                     rare: bool = False) -> None:
         """Decrement reference count of C expression `dest`.
 
         For composite unboxed structures (e.g. tuples) recursively
         decrement reference counts for each component.
+
+        If rare is True, optimize for code size and compilation speed.
         """
         x = 'X' if is_xdec else ''
         if is_int_rprimitive(rtype):
-            self.emit_line('CPyTagged_%sDecRef(%s);' % (x, dest))
+            if rare:
+                self.emit_line('CPyTagged_%sDecRef(%s);' % (x, dest))
+            else:
+                # Inlined
+                self.emit_line('CPyTagged_%sDECREF(%s);' % (x, dest))
         elif isinstance(rtype, RTuple):
             for i, item_type in enumerate(rtype.types):
-                self.emit_dec_ref('{}.f{}'.format(dest, i), item_type, is_xdec)
+                self.emit_dec_ref('{}.f{}'.format(dest, i), item_type, is_xdec=is_xdec, rare=rare)
         elif not rtype.is_unboxed:
-            self.emit_line('CPy_%sDecRef(%s);' % (x, dest))
+            if rare:
+                self.emit_line('CPy_%sDecRef(%s);' % (x, dest))
+            else:
+                # Inlined
+                self.emit_line('CPy_%sDECREF(%s);' % (x, dest))
         # Otherwise assume it's an unboxed, pointerless value and do nothing.
 
     def pretty_name(self, typ: RType) -> str:
@@ -353,8 +406,15 @@ class Emitter:
             return '%s or None' % self.pretty_name(value_type)
         return str(typ)
 
-    def emit_cast(self, src: str, dest: str, typ: RType, declare_dest: bool = False,
-                  custom_message: Optional[str] = None, optional: bool = False,
+    def emit_cast(self,
+                  src: str,
+                  dest: str,
+                  typ: RType,
+                  *,
+                  declare_dest: bool = False,
+                  error: Optional[ErrorHandler] = None,
+                  raise_exception: bool = True,
+                  optional: bool = False,
                   src_type: Optional[RType] = None,
                   likely: bool = True) -> None:
         """Emit code for casting a value of given type.
@@ -363,21 +423,34 @@ class Emitter:
         operates on boxed versions.  This is necessary to properly
         handle types such as Optional[int] in compatibility glue.
 
-        Assign NULL (error value) to dest if the value has an incompatible type.
+        By default, assign NULL (error value) to dest if the value has
+        an incompatible type and raise TypeError. These can be customized
+        using 'error' and 'raise_exception'.
 
-        Always copy/steal the reference in src.
+        Always copy/steal the reference in 'src'.
 
         Args:
             src: Name of source C variable
             dest: Name of target C variable
             typ: Type of value
             declare_dest: If True, also declare the variable 'dest'
+            error: What happens on error
+            raise_exception: If True, also raise TypeError on failure
             likely: If the cast is likely to succeed (can be False for unions)
         """
-        if custom_message is not None:
-            err = custom_message
+        error = error or AssignHandler()
+        if isinstance(error, AssignHandler):
+            handle_error = '%s = NULL;' % dest
+        elif isinstance(error, GotoHandler):
+            handle_error = 'goto %s;' % error.label
         else:
-            err = 'CPy_TypeError("{}", {});'.format(self.pretty_name(typ), src)
+            assert isinstance(error, ReturnHandler)
+            handle_error = 'return %s;' % error.value
+        if raise_exception:
+            raise_exc = 'CPy_TypeError("{}", {}); '.format(self.pretty_name(typ), src)
+            err = raise_exc + handle_error
+        else:
+            err = handle_error
 
         # Special case casting *from* optional
         if src_type and is_optional_type(src_type) and not is_object_rprimitive(typ):
@@ -394,14 +467,13 @@ class Emitter:
                     '    {} = {};'.format(dest, src),
                     'else {',
                     err,
-                    '{} = NULL;'.format(dest),
                     '}')
                 return
 
         # TODO: Verify refcount handling.
         if (is_list_rprimitive(typ) or is_dict_rprimitive(typ) or is_set_rprimitive(typ)
-                or is_float_rprimitive(typ) or is_str_rprimitive(typ) or is_int_rprimitive(typ)
-                or is_bool_rprimitive(typ)):
+                or is_str_rprimitive(typ) or is_range_rprimitive(typ) or is_float_rprimitive(typ)
+                or is_int_rprimitive(typ) or is_bool_rprimitive(typ) or is_bit_rprimitive(typ)):
             if declare_dest:
                 self.emit_line('PyObject *{};'.format(dest))
             if is_list_rprimitive(typ):
@@ -410,10 +482,12 @@ class Emitter:
                 prefix = 'PyDict'
             elif is_set_rprimitive(typ):
                 prefix = 'PySet'
-            elif is_float_rprimitive(typ):
-                prefix = 'CPyFloat'
             elif is_str_rprimitive(typ):
                 prefix = 'PyUnicode'
+            elif is_range_rprimitive(typ):
+                prefix = 'PyRange'
+            elif is_float_rprimitive(typ):
+                prefix = 'CPyFloat'
             elif is_int_rprimitive(typ):
                 prefix = 'PyLong'
             elif is_bool_rprimitive(typ) or is_bit_rprimitive(typ):
@@ -428,7 +502,18 @@ class Emitter:
                 '    {} = {};'.format(dest, src),
                 'else {',
                 err,
-                '{} = NULL;'.format(dest),
+                '}')
+        elif is_bytes_rprimitive(typ):
+            if declare_dest:
+                self.emit_line('PyObject *{};'.format(dest))
+            check = '(PyBytes_Check({}) || PyByteArray_Check({}))'
+            if likely:
+                check = '(likely{})'.format(check)
+            self.emit_arg_check(src, dest, typ, check.format(src, src), optional)
+            self.emit_lines(
+                '    {} = {};'.format(dest, src),
+                'else {',
+                err,
                 '}')
         elif is_tuple_rprimitive(typ):
             if declare_dest:
@@ -442,7 +527,6 @@ class Emitter:
                 '    {} = {};'.format(dest, src),
                 'else {',
                 err,
-                '{} = NULL;'.format(dest),
                 '}')
         elif isinstance(typ, RInstance):
             if declare_dest:
@@ -470,7 +554,6 @@ class Emitter:
                 '    {} = {};'.format(dest, src),
                 'else {',
                 err,
-                '{} = NULL;'.format(dest),
                 '}')
         elif is_none_rprimitive(typ):
             if declare_dest:
@@ -484,7 +567,6 @@ class Emitter:
                 '    {} = {};'.format(dest, src),
                 'else {',
                 err,
-                '{} = NULL;'.format(dest),
                 '}')
         elif is_object_rprimitive(typ):
             if declare_dest:
@@ -501,8 +583,14 @@ class Emitter:
         else:
             assert False, 'Cast not implemented: %s' % typ
 
-    def emit_union_cast(self, src: str, dest: str, typ: RUnion, declare_dest: bool,
-                        err: str, optional: bool, src_type: Optional[RType]) -> None:
+    def emit_union_cast(self,
+                        src: str,
+                        dest: str,
+                        typ: RUnion,
+                        declare_dest: bool,
+                        err: str,
+                        optional: bool,
+                        src_type: Optional[RType]) -> None:
         """Emit cast to a union type.
 
         The arguments are similar to emit_cast.
@@ -520,7 +608,7 @@ class Emitter:
                            dest,
                            item,
                            declare_dest=False,
-                           custom_message='',
+                           raise_exception=False,
                            optional=False,
                            likely=False)
             self.emit_line('if ({} != NULL) goto {};'.format(dest, good_label))
@@ -552,7 +640,7 @@ class Emitter:
                            dest,
                            item,
                            declare_dest=False,
-                           custom_message='',
+                           raise_exception=False,
                            optional=False)
             self.emit_line('if ({} == NULL) goto {};'.format(dest, out_label))
 
@@ -568,31 +656,46 @@ class Emitter:
         elif optional:
             self.emit_line('else {')
 
-    def emit_unbox(self, src: str, dest: str, typ: RType, custom_failure: Optional[str] = None,
-                   declare_dest: bool = False, borrow: bool = False,
-                   optional: bool = False) -> None:
+    def emit_unbox(self,
+                   src: str,
+                   dest: str,
+                   typ: RType,
+                   *,
+                   declare_dest: bool = False,
+                   error: Optional[ErrorHandler] = None,
+                   raise_exception: bool = True,
+                   optional: bool = False,
+                   borrow: bool = False) -> None:
         """Emit code for unboxing a value of given type (from PyObject *).
 
-        Evaluate C code in 'failure' if the value has an incompatible type.
+        By default, assign error value to dest if the value has an
+        incompatible type and raise TypeError. These can be customized
+        using 'error' and 'raise_exception'.
 
-        Always generate a new reference.
+        Generate a new reference unless 'borrow' is True.
 
         Args:
             src: Name of source C variable
             dest: Name of target C variable
             typ: Type of value
-            failure: What happens on error
             declare_dest: If True, also declare the variable 'dest'
+            error: What happens on error
+            raise_exception: If True, also raise TypeError on failure
             borrow: If True, create a borrowed reference
+
         """
+        error = error or AssignHandler()
         # TODO: Verify refcount handling.
-        raise_exc = 'CPy_TypeError("{}", {});'.format(self.pretty_name(typ), src)
-        if custom_failure is not None:
-            failure = [raise_exc,
-                       custom_failure]
+        if isinstance(error, AssignHandler):
+            failure = '%s = %s;' % (dest, self.c_error_value(typ))
+        elif isinstance(error, GotoHandler):
+            failure = 'goto %s;' % error.label
         else:
-            failure = [raise_exc,
-                       '%s = %s;' % (dest, self.c_error_value(typ))]
+            assert isinstance(error, ReturnHandler)
+            failure = 'return %s;' % error.value
+        if raise_exception:
+            raise_exc = 'CPy_TypeError("{}", {}); '.format(self.pretty_name(typ), src)
+            failure = raise_exc + failure
         if is_int_rprimitive(typ) or is_short_int_rprimitive(typ):
             if declare_dest:
                 self.emit_line('CPyTagged {};'.format(dest))
@@ -603,7 +706,7 @@ class Emitter:
             else:
                 self.emit_line('    {} = CPyTagged_FromObject({});'.format(dest, src))
             self.emit_line('else {')
-            self.emit_lines(*failure)
+            self.emit_line(failure)
             self.emit_line('}')
         elif is_bool_rprimitive(typ) or is_bit_rprimitive(typ):
             # Whether we are borrowing or not makes no difference.
@@ -611,7 +714,7 @@ class Emitter:
                 self.emit_line('char {};'.format(dest))
             self.emit_arg_check(src, dest, typ, '(unlikely(!PyBool_Check({}))) {{'.format(src),
                                 optional)
-            self.emit_lines(*failure)
+            self.emit_line(failure)
             self.emit_line('} else')
             conversion = '{} == Py_True'.format(src)
             self.emit_line('    {} = {};'.format(dest, conversion))
@@ -621,7 +724,7 @@ class Emitter:
                 self.emit_line('char {};'.format(dest))
             self.emit_arg_check(src, dest, typ, '(unlikely({} != Py_None)) {{'.format(src),
                                 optional)
-            self.emit_lines(*failure)
+            self.emit_line(failure)
             self.emit_line('} else')
             self.emit_line('    {} = 1;'.format(dest))
         elif isinstance(typ, RTuple):
@@ -643,7 +746,7 @@ class Emitter:
             # self.emit_arg_check(src, dest, typ,
             #     '(!PyTuple_Check({}) || PyTuple_Size({}) != {}) {{'.format(
             #         src, src, len(typ.types)), optional)
-            self.emit_lines(*failure)  # TODO: Decrease refcount?
+            self.emit_line(failure)  # TODO: Decrease refcount?
             self.emit_line('} else {')
             if not typ.types:
                 self.emit_line('{}.empty_struct_error_flag = 0;'.format(dest))
@@ -654,7 +757,12 @@ class Emitter:
                 temp2 = self.temp_name()
                 # Unbox or check the item.
                 if item_type.is_unboxed:
-                    self.emit_unbox(temp, temp2, item_type, custom_failure, declare_dest=True,
+                    self.emit_unbox(temp,
+                                    temp2,
+                                    item_type,
+                                    raise_exception=raise_exception,
+                                    error=error,
+                                    declare_dest=True,
                                     borrow=borrow)
                 else:
                     if not borrow:
