@@ -1,6 +1,6 @@
 """Code generation for native function bodies."""
 
-from typing import Union
+from typing import Union, Optional
 from typing_extensions import Final
 
 from mypyc.common import (
@@ -12,15 +12,16 @@ from mypyc.ir.ops import (
     LoadStatic, InitStatic, TupleGet, TupleSet, Call, IncRef, DecRef, Box, Cast, Unbox,
     BasicBlock, Value, MethodCall, Unreachable, NAMESPACE_STATIC, NAMESPACE_TYPE, NAMESPACE_MODULE,
     RaiseStandardError, CallC, LoadGlobal, Truncate, IntOp, LoadMem, GetElementPtr,
-    LoadAddress, ComparisonOp, SetMem, Register
+    LoadAddress, ComparisonOp, SetMem, Register, LoadLiteral, AssignMulti, KeepAlive
 )
 from mypyc.ir.rtypes import (
-    RType, RTuple, is_tagged, is_int32_rprimitive, is_int64_rprimitive, RStruct,
-    is_pointer_rprimitive
+    RType, RTuple, RArray, is_tagged, is_int32_rprimitive, is_int64_rprimitive, RStruct,
+    is_pointer_rprimitive, is_int_rprimitive
 )
 from mypyc.ir.func_ir import FuncIR, FuncDecl, FUNC_STATICMETHOD, FUNC_CLASSMETHOD, all_values
 from mypyc.ir.class_ir import ClassIR
 from mypyc.ir.pprint import generate_names_for_ir
+from mypyc.analysis.blockfreq import frequently_executed_blocks
 
 # Whether to insert debug asserts for all error handling, to quickly
 # catch errors propagating without exceptions set.
@@ -59,6 +60,8 @@ def generate_native_function(fn: FuncIR,
     for r in all_values(fn.arg_regs, fn.blocks):
         if isinstance(r.type, RTuple):
             emitter.declare_tuple_struct(r.type)
+        if isinstance(r.type, RArray):
+            continue  # Special: declared on first assignment
 
         if r in fn.arg_regs:
             continue  # Skip the arguments
@@ -71,11 +74,20 @@ def generate_native_function(fn: FuncIR,
                                                                      init=init))
 
     # Before we emit the blocks, give them all labels
-    for i, block in enumerate(fn.blocks):
+    blocks = fn.blocks
+    for i, block in enumerate(blocks):
         block.label = i
 
-    for block in fn.blocks:
+    common = frequently_executed_blocks(fn.blocks[0])
+
+    for i in range(len(blocks)):
+        block = blocks[i]
+        visitor.rare = block not in common
+        next_block = None
+        if i + 1 < len(blocks):
+            next_block = blocks[i + 1]
         body.emit_label(block)
+        visitor.next_block = next_block
         for op in block.ops:
             op.accept(visitor)
 
@@ -96,23 +108,35 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         self.declarations = declarations
         self.source_path = source_path
         self.module_name = module_name
+        self.literals = emitter.context.literals
+        self.rare = False
+        self.next_block: Optional[BasicBlock] = None
 
     def temp_name(self) -> str:
         return self.emitter.temp_name()
 
     def visit_goto(self, op: Goto) -> None:
-        self.emit_line('goto %s;' % self.label(op.label))
+        if op.label is not self.next_block:
+            self.emit_line('goto %s;' % self.label(op.label))
 
     def visit_branch(self, op: Branch) -> None:
-        neg = '!' if op.negated else ''
+        true, false = op.true, op.false
+        negated = op.negated
+        negated_rare = False
+        if true is self.next_block and op.traceback_entry is None:
+            # Switch true/false since it avoids an else block.
+            true, false = false, true
+            negated = not negated
+            negated_rare = True
 
+        neg = '!' if negated else ''
         cond = ''
         if op.op == Branch.BOOL:
             expr_result = self.reg(op.value)
             cond = '{}{}'.format(neg, expr_result)
         elif op.op == Branch.IS_ERROR:
             typ = op.value.type
-            compare = '!=' if op.negated else '=='
+            compare = '!=' if negated else '=='
             if isinstance(typ, RTuple):
                 # TODO: What about empty tuple?
                 cond = self.emitter.tuple_undefined_check_cond(typ,
@@ -128,17 +152,29 @@ class FunctionEmitterVisitor(OpVisitor[None]):
 
         # For error checks, tell the compiler the branch is unlikely
         if op.traceback_entry is not None or op.rare:
-            cond = 'unlikely({})'.format(cond)
+            if not negated_rare:
+                cond = 'unlikely({})'.format(cond)
+            else:
+                cond = 'likely({})'.format(cond)
 
-        self.emit_line('if ({}) {{'.format(cond))
-
-        self.emit_traceback(op)
-
-        self.emit_lines(
-            'goto %s;' % self.label(op.true),
-            '} else',
-            '    goto %s;' % self.label(op.false)
-        )
+        if false is self.next_block:
+            if op.traceback_entry is None:
+                self.emit_line('if ({}) goto {};'.format(cond, self.label(true)))
+            else:
+                self.emit_line('if ({}) {{'.format(cond))
+                self.emit_traceback(op)
+                self.emit_lines(
+                    'goto %s;' % self.label(true),
+                    '}'
+                )
+        else:
+            self.emit_line('if ({}) {{'.format(cond))
+            self.emit_traceback(op)
+            self.emit_lines(
+                'goto %s;' % self.label(true),
+                '} else',
+                '    goto %s;' % self.label(false)
+            )
 
     def visit_return(self, op: Return) -> None:
         value_str = self.reg(op.value)
@@ -163,6 +199,18 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         if dest != src:
             self.emit_line('%s = %s;' % (dest, src))
 
+    def visit_assign_multi(self, op: AssignMulti) -> None:
+        typ = op.dest.type
+        assert isinstance(typ, RArray)
+        dest = self.reg(op.dest)
+        # RArray values can only be assigned to once, so we can always
+        # declare them on initialization.
+        self.emit_line('%s%s[%d] = {%s};' % (
+            self.emitter.ctype_spaced(typ.item_type),
+            dest,
+            len(op.src),
+            ', '.join(self.reg(s) for s in op.src)))
+
     def visit_load_error_value(self, op: LoadErrorValue) -> None:
         if isinstance(op.type, RTuple):
             values = [self.c_undefined_value(item) for item in op.type.types]
@@ -173,6 +221,19 @@ class FunctionEmitterVisitor(OpVisitor[None]):
             self.emit_line('%s = %s;' % (self.reg(op),
                                          self.c_error_value(op.type)))
 
+    def visit_load_literal(self, op: LoadLiteral) -> None:
+        index = self.literals.literal_index(op.value)
+        s = repr(op.value)
+        if not any(x in s for x in ('/*', '*/', '\0')):
+            ann = ' /* %s */' % s
+        else:
+            ann = ''
+        if not is_int_rprimitive(op.type):
+            self.emit_line('%s = CPyStatics[%d];%s' % (self.reg(op), index, ann))
+        else:
+            self.emit_line('%s = (CPyTagged)CPyStatics[%d] | 1;%s' % (
+                self.reg(op), index, ann))
+
     def get_attr_expr(self, obj: str, op: Union[GetAttr, SetAttr], decl_cl: ClassIR) -> str:
         """Generate attribute accessor for normal (non-property) access.
 
@@ -231,17 +292,17 @@ class FunctionEmitterVisitor(OpVisitor[None]):
             # Otherwise, use direct or offset struct access.
             attr_expr = self.get_attr_expr(obj, op, decl_cl)
             self.emitter.emit_line('{} = {};'.format(dest, attr_expr))
+            self.emitter.emit_undefined_attr_check(
+                attr_rtype, attr_expr, '==', unlikely=True
+            )
+            exc_class = 'PyExc_AttributeError'
+            self.emitter.emit_line(
+                'PyErr_SetString({}, "attribute {} of {} undefined");'.format(
+                    exc_class, repr(op.attr), repr(cl.name)))
             if attr_rtype.is_refcounted:
-                self.emitter.emit_undefined_attr_check(
-                    attr_rtype, attr_expr, '==', unlikely=True
-                )
-                exc_class = 'PyExc_AttributeError'
-                self.emitter.emit_lines(
-                    'PyErr_SetString({}, "attribute {} of {} undefined");'.format(
-                        exc_class, repr(op.attr), repr(cl.name)),
-                    '} else {')
+                self.emitter.emit_line('} else {')
                 self.emitter.emit_inc_ref(attr_expr, attr_rtype)
-                self.emitter.emit_line('}')
+            self.emitter.emit_line('}')
 
     def visit_set_attr(self, op: SetAttr) -> None:
         dest = self.reg(op)
@@ -276,11 +337,11 @@ class FunctionEmitterVisitor(OpVisitor[None]):
                 '{} = 1;'.format(dest),
             )
 
-    PREFIX_MAP = {
+    PREFIX_MAP: Final = {
         NAMESPACE_STATIC: STATIC_PREFIX,
         NAMESPACE_TYPE: TYPE_PREFIX,
         NAMESPACE_MODULE: MODULE_PREFIX,
-    }  # type: Final
+    }
 
     def visit_load_static(self, op: LoadStatic) -> None:
         dest = self.reg(op)
@@ -332,7 +393,6 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         rtype = op.receiver_type
         class_ir = rtype.class_ir
         name = op.method
-        method_idx = rtype.method_index(name)
         method = rtype.class_ir.get_method(name)
         assert method is not None
 
@@ -355,6 +415,7 @@ class FunctionEmitterVisitor(OpVisitor[None]):
                 dest, lib, NATIVE_PREFIX, method.cname(self.names), args))
         else:
             # Call using vtable.
+            method_idx = rtype.method_index(name)
             self.emit_line('{}CPY_GET_METHOD{}({}, {}, {}, {}, {})({}); /* {} */'.format(
                 dest, version, obj, self.emitter.type_struct_name(rtype.class_ir),
                 method_idx, rtype.struct_name(self.names), mtype, args, op.method))
@@ -365,7 +426,7 @@ class FunctionEmitterVisitor(OpVisitor[None]):
 
     def visit_dec_ref(self, op: DecRef) -> None:
         src = self.reg(op.src)
-        self.emit_dec_ref(src, op.src.type, op.is_xdec)
+        self.emit_dec_ref(src, op.src.type, is_xdec=op.is_xdec)
 
     def visit_box(self, op: Box) -> None:
         self.emitter.emit_box(self.reg(op.src), self.reg(op), op.src.type, can_borrow=True)
@@ -432,14 +493,20 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         rhs = self.reg(op.rhs)
         lhs_cast = ""
         rhs_cast = ""
-        signed_op = {ComparisonOp.SLT, ComparisonOp.SGT, ComparisonOp.SLE, ComparisonOp.SGE}
-        unsigned_op = {ComparisonOp.ULT, ComparisonOp.UGT, ComparisonOp.ULE, ComparisonOp.UGE}
-        if op.op in signed_op:
+        if op.op in (ComparisonOp.SLT, ComparisonOp.SGT, ComparisonOp.SLE, ComparisonOp.SGE):
+            # Always signed comparison op
             lhs_cast = self.emit_signed_int_cast(op.lhs.type)
             rhs_cast = self.emit_signed_int_cast(op.rhs.type)
-        elif op.op in unsigned_op:
+        elif op.op in (ComparisonOp.ULT, ComparisonOp.UGT, ComparisonOp.ULE, ComparisonOp.UGE):
+            # Always unsigned comparison op
             lhs_cast = self.emit_unsigned_int_cast(op.lhs.type)
             rhs_cast = self.emit_unsigned_int_cast(op.rhs.type)
+        elif isinstance(op.lhs, Integer) and op.lhs.value < 0:
+            # Force signed ==/!= with negative operand
+            rhs_cast = self.emit_signed_int_cast(op.rhs.type)
+        elif isinstance(op.rhs, Integer) and op.rhs.value < 0:
+            # Force signed ==/!= with negative operand
+            lhs_cast = self.emit_signed_int_cast(op.lhs.type)
         self.emit_line('%s = %s%s %s %s%s;' % (dest, lhs_cast, lhs,
                                                op.op_str[op.op], rhs_cast, rhs))
 
@@ -474,6 +541,10 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         src = self.reg(op.src) if isinstance(op.src, Register) else op.src
         self.emit_line('%s = (%s)&%s;' % (dest, typ._ctype, src))
 
+    def visit_keep_alive(self, op: KeepAlive) -> None:
+        # This is a no-op.
+        pass
+
     # Helpers
 
     def label(self, label: BasicBlock) -> str:
@@ -484,7 +555,16 @@ class FunctionEmitterVisitor(OpVisitor[None]):
             val = reg.value
             if val == 0 and is_pointer_rprimitive(reg.type):
                 return "NULL"
-            return str(val)
+            s = str(val)
+            if val >= (1 << 31):
+                # Avoid overflowing signed 32-bit int
+                s += 'ULL'
+            elif val == -(1 << 63):
+                # Avoid overflowing C integer literal
+                s = '(-9223372036854775807LL - 1)'
+            elif val <= -(1 << 31):
+                s += 'LL'
+            return s
         else:
             return self.emitter.reg(reg)
 
@@ -504,10 +584,10 @@ class FunctionEmitterVisitor(OpVisitor[None]):
         self.emitter.emit_lines(*lines)
 
     def emit_inc_ref(self, dest: str, rtype: RType) -> None:
-        self.emitter.emit_inc_ref(dest, rtype)
+        self.emitter.emit_inc_ref(dest, rtype, rare=self.rare)
 
     def emit_dec_ref(self, dest: str, rtype: RType, is_xdec: bool) -> None:
-        self.emitter.emit_dec_ref(dest, rtype, is_xdec)
+        self.emitter.emit_dec_ref(dest, rtype, is_xdec=is_xdec, rare=self.rare)
 
     def emit_declaration(self, line: str) -> None:
         self.declarations.emit_line(line)
