@@ -10,29 +10,36 @@ as an environment containing non-local variables, is stored in the
 instance of the callable class.
 """
 
-from typing import Optional, List, Tuple, Union, Dict
+from typing import (
+    DefaultDict, NamedTuple, Optional, List, Sequence, Tuple, Union, Dict,
+)
 
 from mypy.nodes import (
     ClassDef, FuncDef, OverloadedFuncDef, Decorator, Var, YieldFromExpr, AwaitExpr, YieldExpr,
-    FuncItem, LambdaExpr, SymbolNode
+    FuncItem, LambdaExpr, SymbolNode, ArgKind, TypeInfo
 )
 from mypy.types import CallableType, get_proper_type
 
 from mypyc.ir.ops import (
     BasicBlock, Value,  Register, Return, SetAttr, Integer, GetAttr, Branch, InitStatic,
-    LoadAddress
+    LoadAddress, LoadLiteral, Unbox, Unreachable,
+)
+from mypyc.ir.rtypes import (
+    object_rprimitive, RInstance, object_pointer_rprimitive, dict_rprimitive, int_rprimitive,
+    bool_rprimitive,
 )
-from mypyc.ir.rtypes import object_rprimitive, RInstance, object_pointer_rprimitive
 from mypyc.ir.func_ir import (
     FuncIR, FuncSignature, RuntimeArg, FuncDecl, FUNC_CLASSMETHOD, FUNC_STATICMETHOD, FUNC_NORMAL
 )
 from mypyc.ir.class_ir import ClassIR, NonExtClassInfo
 from mypyc.primitives.generic_ops import py_setattr_op, next_raw_op, iter_op
-from mypyc.primitives.misc_ops import check_stop_op, yield_from_except_op, coro_op, send_op
-from mypyc.primitives.dict_ops import dict_set_item_op
-from mypyc.common import SELF_NAME, LAMBDA_NAME, decorator_helper_name
+from mypyc.primitives.misc_ops import (
+    check_stop_op, yield_from_except_op, coro_op, send_op, register_function
+)
+from mypyc.primitives.dict_ops import dict_set_item_op, dict_new_op, dict_get_method_with_none
+from mypyc.common import SELF_NAME, LAMBDA_NAME
 from mypyc.sametype import is_same_method_signature
-from mypyc.irbuild.util import concrete_arg_kind, is_constant
+from mypyc.irbuild.util import is_constant
 from mypyc.irbuild.context import FuncInfo, ImplicitClass
 from mypyc.irbuild.targets import AssignmentTarget
 from mypyc.irbuild.statement import transform_try_except
@@ -51,6 +58,9 @@ from mypyc.irbuild.env_class import (
     setup_func_for_recursive_call
 )
 
+from mypyc.primitives.registry import builtin_names
+from collections import defaultdict
+
 
 # Top-level transform functions
 
@@ -62,6 +72,7 @@ def transform_func_def(builder: IRBuilder, fdef: FuncDef) -> None:
     # current environment or define it if it was not already defined.
     if func_reg:
         builder.assign(get_func_target(builder, fdef), func_reg, fdef.line)
+    maybe_insert_into_registry_dict(builder, fdef)
     builder.functions.append(func_ir)
 
 
@@ -78,40 +89,34 @@ def transform_decorator(builder: IRBuilder, dec: Decorator) -> None:
         dec.func.name,
         builder.mapper.fdef_to_sig(dec.func)
     )
-
-    if dec.func in builder.nested_fitems:
-        assert func_reg is not None
+    decorated_func: Optional[Value] = None
+    if func_reg:
         decorated_func = load_decorated_func(builder, dec.func, func_reg)
         builder.assign(get_func_target(builder, dec.func), decorated_func, dec.func.line)
         func_reg = decorated_func
-    else:
+    # If the prebuild pass didn't put this function in the function to decorators map (for example
+    # if this is a registered singledispatch implementation with no other decorators), we should
+    # treat this function as a regular function, not a decorated function
+    elif dec.func in builder.fdefs_to_decorators:
         # Obtain the the function name in order to construct the name of the helper function.
         name = dec.func.fullname.split('.')[-1]
-        helper_name = decorator_helper_name(name)
 
         # Load the callable object representing the non-decorated function, and decorate it.
-        orig_func = builder.load_global_str(helper_name, dec.line)
+        orig_func = builder.load_global_str(name, dec.line)
         decorated_func = load_decorated_func(builder, dec.func, orig_func)
 
+    if decorated_func is not None:
         # Set the callable object representing the decorated function as a global.
         builder.call_c(dict_set_item_op,
-                    [builder.load_globals_dict(),
-                    builder.load_static_unicode(dec.func.name), decorated_func],
-                    decorated_func.line)
+                       [builder.load_globals_dict(),
+                        builder.load_str(dec.func.name), decorated_func],
+                       decorated_func.line)
+
+    maybe_insert_into_registry_dict(builder, dec.func)
 
     builder.functions.append(func_ir)
 
 
-def transform_method(builder: IRBuilder,
-                     cdef: ClassDef,
-                     non_ext: Optional[NonExtClassInfo],
-                     fdef: FuncDef) -> None:
-    if non_ext:
-        handle_non_ext_method(builder, non_ext, cdef, fdef)
-    else:
-        handle_ext_method(builder, cdef, fdef)
-
-
 def transform_lambda_expr(builder: IRBuilder, expr: LambdaExpr) -> Value:
     typ = get_proper_type(builder.types[expr])
     assert isinstance(typ, CallableType)
@@ -135,6 +140,9 @@ def transform_lambda_expr(builder: IRBuilder, expr: LambdaExpr) -> Value:
 
 
 def transform_yield_expr(builder: IRBuilder, expr: YieldExpr) -> Value:
+    if builder.fn_info.is_coroutine:
+        builder.error('async generators are unimplemented', expr.line)
+
     if expr.expr:
         retval = builder.accept(expr.expr)
     else:
@@ -195,7 +203,7 @@ def gen_func_item(builder: IRBuilder,
 
     # TODO: do something about abstract methods.
 
-    func_reg = None  # type: Optional[Value]
+    func_reg: Optional[Value] = None
 
     # We treat lambdas as always being nested because we always generate
     # a class for lambdas, no matter where they are. (It would probably also
@@ -203,6 +211,7 @@ def gen_func_item(builder: IRBuilder,
     is_nested = fitem in builder.nested_fitems or isinstance(fitem, LambdaExpr)
     contains_nested = fitem in builder.encapsulating_funcs.keys()
     is_decorated = fitem in builder.fdefs_to_decorators
+    is_singledispatch = fitem in builder.singledispatch_impls
     in_non_ext = False
     class_name = None
     if cdef:
@@ -210,7 +219,11 @@ def gen_func_item(builder: IRBuilder,
         in_non_ext = not ir.is_ext_class
         class_name = cdef.name
 
-    builder.enter(FuncInfo(fitem, name, class_name, gen_func_ns(builder),
+    if is_singledispatch:
+        func_name = singledispatch_main_func_name(name)
+    else:
+        func_name = name
+    builder.enter(FuncInfo(fitem, func_name, class_name, gen_func_ns(builder),
                            is_nested, contains_nested, is_decorated, in_non_ext))
 
     # Functions that contain nested functions need an environment class to store variables that
@@ -227,7 +240,9 @@ def gen_func_item(builder: IRBuilder,
         # Do a first-pass and generate a function that just returns a generator object.
         gen_generator_func(builder)
         args, _, blocks, ret_type, fn_info = builder.leave()
-        func_ir, func_reg = gen_func_ir(builder, args, blocks, sig, fn_info, cdef)
+        func_ir, func_reg = gen_func_ir(
+            builder, args, blocks, sig, fn_info, cdef, is_singledispatch,
+        )
 
         # Re-enter the FuncItem and visit the body of the function this time.
         builder.enter(fn_info)
@@ -252,7 +267,7 @@ def gen_func_item(builder: IRBuilder,
     # them even if they are declared after the nested function's definition.
     # Note that this is done before visiting the body of this function.
 
-    env_for_func = builder.fn_info  # type: Union[FuncInfo, ImplicitClass]
+    env_for_func: Union[FuncInfo, ImplicitClass] = builder.fn_info
     if builder.fn_info.is_generator:
         env_for_func = builder.fn_info.generator_class
     elif builder.fn_info.is_nested or builder.fn_info.in_non_ext:
@@ -294,13 +309,22 @@ def gen_func_item(builder: IRBuilder,
         add_methods_to_generator_class(
             builder, fn_info, sig, args, blocks, fitem.is_coroutine)
     else:
-        func_ir, func_reg = gen_func_ir(builder, args, blocks, sig, fn_info, cdef)
+        func_ir, func_reg = gen_func_ir(
+            builder, args, blocks, sig, fn_info, cdef, is_singledispatch,
+        )
 
     # Evaluate argument defaults in the surrounding scope, since we
     # calculate them *once* when the function definition is evaluated.
     calculate_arg_defaults(builder, fn_info, func_reg, symtable)
 
-    return (func_ir, func_reg)
+    if is_singledispatch:
+        # add the generated main singledispatch function
+        builder.functions.append(func_ir)
+        # create the dispatch function
+        assert isinstance(fitem, FuncDef)
+        return gen_dispatch_func_ir(builder, fitem, fn_info.name, name, sig)
+
+    return func_ir, func_reg
 
 
 def gen_func_ir(builder: IRBuilder,
@@ -308,7 +332,8 @@ def gen_func_ir(builder: IRBuilder,
                 blocks: List[BasicBlock],
                 sig: FuncSignature,
                 fn_info: FuncInfo,
-                cdef: Optional[ClassDef]) -> Tuple[FuncIR, Optional[Value]]:
+                cdef: Optional[ClassDef],
+                is_singledispatch_main_func: bool = False) -> Tuple[FuncIR, Optional[Value]]:
     """Generate the FuncIR for a function.
 
     This takes the basic blocks and function info of a particular
@@ -316,7 +341,7 @@ def gen_func_ir(builder: IRBuilder,
     also returns the register containing the instance of the
     corresponding callable class.
     """
-    func_reg = None  # type: Optional[Value]
+    func_reg: Optional[Value] = None
     if fn_info.is_nested or fn_info.in_non_ext:
         func_ir = add_call_to_callable_class(builder, args, blocks, sig, fn_info)
         add_get_to_callable_class(builder, fn_info)
@@ -324,7 +349,7 @@ def gen_func_ir(builder: IRBuilder,
     else:
         assert isinstance(fn_info.fitem, FuncDef)
         func_decl = builder.mapper.func_to_decl[fn_info.fitem]
-        if fn_info.is_decorated:
+        if fn_info.is_decorated or is_singledispatch_main_func:
             class_name = None if cdef is None else cdef.name
             func_decl = FuncDecl(fn_info.name, class_name, builder.module_name, sig,
                                  func_decl.kind,
@@ -347,11 +372,10 @@ def handle_ext_method(builder: IRBuilder, cdef: ClassDef, fdef: FuncDef) -> None
     if is_decorated(builder, fdef):
         # Obtain the the function name in order to construct the name of the helper function.
         _, _, name = fdef.fullname.rpartition('.')
-        helper_name = decorator_helper_name(name)
         # Read the PyTypeObject representing the class, get the callable object
         # representing the non-decorated method
         typ = builder.load_native_type_object(cdef.fullname)
-        orig_func = builder.py_get_attr(typ, helper_name, fdef.line)
+        orig_func = builder.py_get_attr(typ, name, fdef.line)
 
         # Decorate the non-decorated method
         decorated_func = load_decorated_func(builder, fdef, orig_func)
@@ -359,12 +383,8 @@ def handle_ext_method(builder: IRBuilder, cdef: ClassDef, fdef: FuncDef) -> None
         # Set the callable object representing the decorated method as an attribute of the
         # extension class.
         builder.call_c(py_setattr_op,
-                    [
-                        typ,
-                        builder.load_static_unicode(name),
-                        decorated_func
-                    ],
-                    fdef.line)
+                       [typ, builder.load_str(name), decorated_func],
+                       fdef.line)
 
     if fdef.is_property:
         # If there is a property setter, it will be processed after the getter,
@@ -625,6 +645,24 @@ def gen_glue(builder: IRBuilder, sig: FuncSignature, target: FuncIR,
         return gen_glue_method(builder, sig, target, cls, base, fdef.line, do_py_ops)
 
 
+class ArgInfo(NamedTuple):
+    args: List[Value]
+    arg_names: List[Optional[str]]
+    arg_kinds: List[ArgKind]
+
+
+def get_args(builder: IRBuilder, rt_args: Sequence[RuntimeArg], line: int) -> ArgInfo:
+    # The environment operates on Vars, so we make some up
+    fake_vars = [(Var(arg.name), arg.type) for arg in rt_args]
+    args = [builder.read(builder.add_local_reg(var, type, is_arg=True), line)
+            for var, type in fake_vars]
+    arg_names = [arg.name
+                 if arg.kind.is_named() or (arg.kind.is_optional() and not arg.pos_only) else None
+                 for arg in rt_args]
+    arg_kinds = [arg.kind for arg in rt_args]
+    return ArgInfo(args, arg_names, arg_kinds)
+
+
 def gen_glue_method(builder: IRBuilder, sig: FuncSignature, target: FuncIR,
                     cls: ClassIR, base: ClassIR, line: int,
                     do_pycall: bool,
@@ -661,16 +699,27 @@ def gen_glue_method(builder: IRBuilder, sig: FuncSignature, target: FuncIR,
     if target.decl.kind == FUNC_NORMAL:
         rt_args[0] = RuntimeArg(sig.args[0].name, RInstance(cls))
 
-    # The environment operates on Vars, so we make some up
-    fake_vars = [(Var(arg.name), arg.type) for arg in rt_args]
-    args = [builder.read(builder.add_local_reg(var, type, is_arg=True), line)
-            for var, type in fake_vars]
-    arg_names = [arg.name for arg in rt_args]
-    arg_kinds = [concrete_arg_kind(arg.kind) for arg in rt_args]
+    arg_info = get_args(builder, rt_args, line)
+    args, arg_kinds, arg_names = arg_info.args, arg_info.arg_kinds, arg_info.arg_names
+
+    # We can do a passthrough *args/**kwargs with a native call, but if the
+    # args need to get distributed out to arguments, we just let python handle it
+    if (
+        any(kind.is_star() for kind in arg_kinds)
+        and any(not arg.kind.is_star() for arg in target.decl.sig.args)
+    ):
+        do_pycall = True
 
     if do_pycall:
+        if target.decl.kind == FUNC_STATICMETHOD:
+            # FIXME: this won't work if we can do interpreted subclasses
+            first = builder.builder.get_native_type(cls)
+            st = 0
+        else:
+            first = args[0]
+            st = 1
         retval = builder.builder.py_method_call(
-            args[0], target.name, args[1:], line, arg_kinds[1:], arg_names[1:])
+            first, target.name, args[st:], line, arg_kinds[st:], arg_names[st:])
     else:
         retval = builder.builder.call(target.decl, args, arg_kinds, arg_names, line)
     retval = builder.coerce(retval, sig.ret_type, line)
@@ -735,3 +784,274 @@ def get_func_target(builder: IRBuilder, fdef: FuncDef) -> AssignmentTarget:
         return builder.lookup(fdef)
 
     return builder.add_local_reg(fdef, object_rprimitive)
+
+
+def load_type(builder: IRBuilder, typ: TypeInfo, line: int) -> Value:
+    if typ in builder.mapper.type_to_ir:
+        class_ir = builder.mapper.type_to_ir[typ]
+        class_obj = builder.builder.get_native_type(class_ir)
+    elif typ.fullname in builtin_names:
+        builtin_addr_type, src = builtin_names[typ.fullname]
+        class_obj = builder.add(LoadAddress(builtin_addr_type, src, line))
+    else:
+        class_obj = builder.load_global_str(typ.name, line)
+
+    return class_obj
+
+
+def load_func(builder: IRBuilder, func_name: str, fullname: Optional[str], line: int) -> Value:
+    if fullname is not None and not fullname.startswith(builder.current_module):
+        # we're calling a function in a different module
+
+        # We can't use load_module_attr_by_fullname here because we need to load the function using
+        # func_name, not the name specified by fullname (which can be different for underscore
+        # function)
+        module = fullname.rsplit('.')[0]
+        loaded_module = builder.load_module(module)
+
+        func = builder.py_get_attr(loaded_module, func_name, line)
+    else:
+        func = builder.load_global_str(func_name, line)
+    return func
+
+
+def generate_singledispatch_dispatch_function(
+    builder: IRBuilder,
+    main_singledispatch_function_name: str,
+    fitem: FuncDef,
+) -> None:
+    line = fitem.line
+    current_func_decl = builder.mapper.func_to_decl[fitem]
+    arg_info = get_args(builder, current_func_decl.sig.args, line)
+
+    dispatch_func_obj = builder.self()
+
+    arg_type = builder.builder.get_type_of_obj(arg_info.args[0], line)
+    dispatch_cache = builder.builder.get_attr(
+        dispatch_func_obj, 'dispatch_cache', dict_rprimitive, line
+    )
+    call_find_impl, use_cache, call_func = BasicBlock(), BasicBlock(), BasicBlock()
+    get_result = builder.call_c(dict_get_method_with_none, [dispatch_cache, arg_type], line)
+    is_not_none = builder.translate_is_op(get_result, builder.none_object(), 'is not', line)
+    impl_to_use = Register(object_rprimitive)
+    builder.add_bool_branch(is_not_none, use_cache, call_find_impl)
+
+    builder.activate_block(use_cache)
+    builder.assign(impl_to_use, get_result, line)
+    builder.goto(call_func)
+
+    builder.activate_block(call_find_impl)
+    find_impl = builder.load_module_attr_by_fullname('functools._find_impl', line)
+    registry = load_singledispatch_registry(builder, dispatch_func_obj, line)
+    uncached_impl = builder.py_call(find_impl, [arg_type, registry], line)
+    builder.call_c(dict_set_item_op, [dispatch_cache, arg_type, uncached_impl], line)
+    builder.assign(impl_to_use, uncached_impl, line)
+    builder.goto(call_func)
+
+    builder.activate_block(call_func)
+    gen_calls_to_correct_impl(builder, impl_to_use, arg_info, fitem, line)
+
+
+def gen_calls_to_correct_impl(
+    builder: IRBuilder,
+    impl_to_use: Value,
+    arg_info: ArgInfo,
+    fitem: FuncDef,
+    line: int,
+) -> None:
+    current_func_decl = builder.mapper.func_to_decl[fitem]
+
+    def gen_native_func_call_and_return(fdef: FuncDef) -> None:
+        func_decl = builder.mapper.func_to_decl[fdef]
+        ret_val = builder.builder.call(
+            func_decl, arg_info.args, arg_info.arg_kinds, arg_info.arg_names, line
+        )
+        coerced = builder.coerce(ret_val, current_func_decl.sig.ret_type, line)
+        builder.add(Return(coerced))
+
+    typ, src = builtin_names['builtins.int']
+    int_type_obj = builder.add(LoadAddress(typ, src, line))
+    is_int = builder.builder.type_is_op(impl_to_use, int_type_obj, line)
+
+    native_call, non_native_call = BasicBlock(), BasicBlock()
+    builder.add_bool_branch(is_int, native_call, non_native_call)
+    builder.activate_block(native_call)
+
+    passed_id = builder.add(Unbox(impl_to_use, int_rprimitive, line))
+
+    native_ids = get_native_impl_ids(builder, fitem)
+    for impl, i in native_ids.items():
+        call_impl, next_impl = BasicBlock(), BasicBlock()
+
+        current_id = builder.load_int(i)
+        builder.builder.compare_tagged_condition(
+            passed_id,
+            current_id,
+            '==',
+            call_impl,
+            next_impl,
+            line,
+        )
+
+        # Call the registered implementation
+        builder.activate_block(call_impl)
+
+        gen_native_func_call_and_return(impl)
+        builder.activate_block(next_impl)
+
+    # We've already handled all the possible integer IDs, so we should never get here
+    builder.add(Unreachable())
+
+    builder.activate_block(non_native_call)
+    ret_val = builder.py_call(
+        impl_to_use, arg_info.args, line, arg_info.arg_kinds, arg_info.arg_names
+    )
+    coerced = builder.coerce(ret_val, current_func_decl.sig.ret_type, line)
+    builder.add(Return(coerced))
+
+
+def gen_dispatch_func_ir(
+    builder: IRBuilder,
+    fitem: FuncDef,
+    main_func_name: str,
+    dispatch_name: str,
+    sig: FuncSignature,
+) -> Tuple[FuncIR, Value]:
+    """Create a dispatch function (a function that checks the first argument type and dispatches
+    to the correct implementation)
+    """
+    builder.enter(FuncInfo(fitem, dispatch_name))
+    setup_callable_class(builder)
+    builder.fn_info.callable_class.ir.attributes['registry'] = dict_rprimitive
+    builder.fn_info.callable_class.ir.attributes['dispatch_cache'] = dict_rprimitive
+    builder.fn_info.callable_class.ir.has_dict = True
+    builder.fn_info.callable_class.ir.needs_getseters = True
+    generate_singledispatch_callable_class_ctor(builder)
+
+    generate_singledispatch_dispatch_function(builder, main_func_name, fitem)
+    args, _, blocks, _, fn_info = builder.leave()
+    dispatch_callable_class = add_call_to_callable_class(builder, args, blocks, sig, fn_info)
+    builder.functions.append(dispatch_callable_class)
+    add_get_to_callable_class(builder, fn_info)
+    add_register_method_to_callable_class(builder, fn_info)
+    func_reg = instantiate_callable_class(builder, fn_info)
+    dispatch_func_ir = generate_dispatch_glue_native_function(
+        builder, fitem, dispatch_callable_class.decl, dispatch_name
+    )
+
+    return dispatch_func_ir, func_reg
+
+
+def generate_dispatch_glue_native_function(
+    builder: IRBuilder,
+    fitem: FuncDef,
+    callable_class_decl: FuncDecl,
+    dispatch_name: str,
+) -> FuncIR:
+    line = fitem.line
+    builder.enter()
+    # We store the callable class in the globals dict for this function
+    callable_class = builder.load_global_str(dispatch_name, line)
+    decl = builder.mapper.func_to_decl[fitem]
+    arg_info = get_args(builder, decl.sig.args, line)
+    args = [callable_class] + arg_info.args
+    arg_kinds = [ArgKind.ARG_POS] + arg_info.arg_kinds
+    arg_names = arg_info.arg_names
+    arg_names.insert(0, 'self')
+    ret_val = builder.builder.call(callable_class_decl, args, arg_kinds, arg_names, line)
+    builder.add(Return(ret_val))
+    arg_regs, _, blocks, _, fn_info = builder.leave()
+    return FuncIR(decl, arg_regs, blocks)
+
+
+def generate_singledispatch_callable_class_ctor(builder: IRBuilder) -> None:
+    """Create an __init__ that sets registry and dispatch_cache to empty dicts"""
+    line = -1
+    class_ir = builder.fn_info.callable_class.ir
+    with builder.enter_method(class_ir, '__init__', bool_rprimitive):
+        empty_dict = builder.call_c(dict_new_op, [], line)
+        builder.add(SetAttr(builder.self(), 'registry', empty_dict, line))
+        cache_dict = builder.call_c(dict_new_op, [], line)
+        dispatch_cache_str = builder.load_str('dispatch_cache')
+        # use the py_setattr_op instead of SetAttr so that it also gets added to our __dict__
+        builder.call_c(py_setattr_op, [builder.self(), dispatch_cache_str, cache_dict], line)
+        # the generated C code seems to expect that __init__ returns a char, so just return 1
+        builder.add(Return(Integer(1, bool_rprimitive, line), line))
+
+
+def add_register_method_to_callable_class(builder: IRBuilder, fn_info: FuncInfo) -> None:
+    line = -1
+    with builder.enter_method(fn_info.callable_class.ir, 'register', object_rprimitive):
+        cls_arg = builder.add_argument('cls', object_rprimitive)
+        func_arg = builder.add_argument('func', object_rprimitive, ArgKind.ARG_OPT)
+        ret_val = builder.call_c(register_function, [builder.self(), cls_arg, func_arg], line)
+        builder.add(Return(ret_val, line))
+
+
+def load_singledispatch_registry(builder: IRBuilder, dispatch_func_obj: Value, line: int) -> Value:
+    return builder.builder.get_attr(dispatch_func_obj, 'registry', dict_rprimitive, line)
+
+
+def singledispatch_main_func_name(orig_name: str) -> str:
+    return '__mypyc_singledispatch_main_function_{}__'.format(orig_name)
+
+
+def get_registry_identifier(fitem: FuncDef) -> str:
+    return f'__mypyc_singledispatch_registry_{fitem.fullname}__'
+
+
+def maybe_insert_into_registry_dict(builder: IRBuilder, fitem: FuncDef) -> None:
+    line = fitem.line
+    is_singledispatch_main_func = fitem in builder.singledispatch_impls
+    # dict of singledispatch_func to list of register_types (fitem is the function to register)
+    to_register: DefaultDict[FuncDef, List[TypeInfo]] = defaultdict(list)
+    for main_func, impls in builder.singledispatch_impls.items():
+        for dispatch_type, impl in impls:
+            if fitem == impl:
+                to_register[main_func].append(dispatch_type)
+
+    if not to_register and not is_singledispatch_main_func:
+        return
+
+    if is_singledispatch_main_func:
+        main_func_name = singledispatch_main_func_name(fitem.name)
+        main_func_obj = load_func(builder, main_func_name, fitem.fullname, line)
+
+        loaded_object_type = builder.load_module_attr_by_fullname('builtins.object', line)
+        registry_dict = builder.builder.make_dict([(loaded_object_type, main_func_obj)], line)
+
+        dispatch_func_obj = builder.load_global_str(fitem.name, line)
+        builder.call_c(
+            py_setattr_op, [dispatch_func_obj, builder.load_str('registry'), registry_dict], line
+        )
+
+    for singledispatch_func, types in to_register.items():
+        # TODO: avoid recomputing the native IDs for all the functions every time we find a new
+        # function
+        native_ids = get_native_impl_ids(builder, singledispatch_func)
+        if fitem not in native_ids:
+            to_insert = load_func(builder, fitem.name, fitem.fullname, line)
+        else:
+            current_id = native_ids[fitem]
+            load_literal = LoadLiteral(current_id, object_rprimitive)
+            to_insert = builder.add(load_literal)
+        # TODO: avoid reloading the registry here if we just created it
+        dispatch_func_obj = load_func(
+            builder, singledispatch_func.name, singledispatch_func.fullname, line
+        )
+        registry = load_singledispatch_registry(builder, dispatch_func_obj, line)
+        for typ in types:
+            loaded_type = load_type(builder, typ, line)
+            builder.call_c(dict_set_item_op, [registry, loaded_type, to_insert], line)
+        dispatch_cache = builder.builder.get_attr(
+            dispatch_func_obj, 'dispatch_cache', dict_rprimitive, line
+        )
+        builder.gen_method_call(dispatch_cache, 'clear', [], None, line)
+
+
+def get_native_impl_ids(builder: IRBuilder, singledispatch_func: FuncDef) -> Dict[FuncDef, int]:
+    """Return a dict of registered implementation to native implementation ID for all
+    implementations
+    """
+    impls = builder.singledispatch_impls[singledispatch_func]
+    return {impl: i for i, (typ, impl) in enumerate(impls) if not is_decorated(builder, impl)}
