@@ -12,22 +12,25 @@ import importlib.util
 from mypy.nodes import (
     Block, ExpressionStmt, ReturnStmt, AssignmentStmt, OperatorAssignmentStmt, IfStmt, WhileStmt,
     ForStmt, BreakStmt, ContinueStmt, RaiseStmt, TryStmt, WithStmt, AssertStmt, DelStmt,
-    Expression, StrExpr, TempNode, Lvalue, Import, ImportFrom, ImportAll
+    Expression, StrExpr, TempNode, Lvalue, Import, ImportFrom, ImportAll, TupleExpr, ListExpr,
+    StarExpr
 )
 
 from mypyc.ir.ops import (
-    Assign, Unreachable, AssignmentTarget, AssignmentTargetRegister, AssignmentTargetIndex,
-    AssignmentTargetAttr, AssignmentTargetTuple, PrimitiveOp, RaiseStandardError, LoadErrorValue,
-    BasicBlock, TupleGet, Value, Register, Branch, NO_TRACEBACK_LINE_NO
+    Assign, Unreachable, RaiseStandardError, LoadErrorValue, BasicBlock, TupleGet, Value, Register,
+    Branch, NO_TRACEBACK_LINE_NO
 )
-from mypyc.ir.rtypes import exc_rtuple
+from mypyc.ir.rtypes import RInstance, exc_rtuple
 from mypyc.primitives.generic_ops import py_delattr_op
-from mypyc.primitives.misc_ops import true_op, false_op, type_op, get_module_dict_op
-from mypyc.primitives.dict_ops import dict_get_item_op
+from mypyc.primitives.misc_ops import type_op, import_from_op
 from mypyc.primitives.exc_ops import (
     raise_exception_op, reraise_exception_op, error_catch_op, exc_matches_op, restore_exc_info_op,
     get_exc_value_op, keep_propagating_op, get_exc_info_op
 )
+from mypyc.irbuild.targets import (
+    AssignmentTarget, AssignmentTargetRegister, AssignmentTargetIndex, AssignmentTargetAttr,
+    AssignmentTargetTuple
+)
 from mypyc.irbuild.nonlocalcontrol import (
     ExceptNonlocalControl, FinallyNonlocalControl, TryFinallyNonlocalControl
 )
@@ -69,25 +72,47 @@ def transform_return_stmt(builder: IRBuilder, stmt: ReturnStmt) -> None:
 
 
 def transform_assignment_stmt(builder: IRBuilder, stmt: AssignmentStmt) -> None:
-    assert len(stmt.lvalues) >= 1
-    builder.disallow_class_assignments(stmt.lvalues, stmt.line)
-    lvalue = stmt.lvalues[0]
+    lvalues = stmt.lvalues
+    assert len(lvalues) >= 1
+    builder.disallow_class_assignments(lvalues, stmt.line)
+    first_lvalue = lvalues[0]
     if stmt.type and isinstance(stmt.rvalue, TempNode):
         # This is actually a variable annotation without initializer. Don't generate
         # an assignment but we need to call get_assignment_target since it adds a
         # name binding as a side effect.
-        builder.get_assignment_target(lvalue, stmt.line)
+        builder.get_assignment_target(first_lvalue, stmt.line)
+        return
+
+    # Special case multiple assignments like 'x, y = e1, e2'.
+    if (isinstance(first_lvalue, (TupleExpr, ListExpr))
+            and isinstance(stmt.rvalue, (TupleExpr, ListExpr))
+            and len(first_lvalue.items) == len(stmt.rvalue.items)
+            and all(is_simple_lvalue(item) for item in first_lvalue.items)
+            and len(lvalues) == 1):
+        temps = []
+        for right in stmt.rvalue.items:
+            rvalue_reg = builder.accept(right)
+            temp = Register(rvalue_reg.type)
+            builder.assign(temp, rvalue_reg, stmt.line)
+            temps.append(temp)
+        for (left, temp) in zip(first_lvalue.items, temps):
+            assignment_target = builder.get_assignment_target(left)
+            builder.assign(assignment_target, temp, stmt.line)
         return
 
     line = stmt.rvalue.line
     rvalue_reg = builder.accept(stmt.rvalue)
     if builder.non_function_scope() and stmt.is_final_def:
-        builder.init_final_static(lvalue, rvalue_reg)
-    for lvalue in stmt.lvalues:
+        builder.init_final_static(first_lvalue, rvalue_reg)
+    for lvalue in lvalues:
         target = builder.get_assignment_target(lvalue)
         builder.assign(target, rvalue_reg, line)
 
 
+def is_simple_lvalue(expr: Expression) -> bool:
+    return not isinstance(expr, (StarExpr, ListExpr, TupleExpr))
+
+
 def transform_operator_assignment_stmt(builder: IRBuilder, stmt: OperatorAssignmentStmt) -> None:
     """Operator assignment statement such as x += 1"""
     builder.disallow_class_assignments([stmt.lvalue], stmt.line)
@@ -117,6 +142,8 @@ def transform_import(builder: IRBuilder, node: Import) -> None:
         # that mypy couldn't find, since it doesn't analyze module references
         # from those properly.
 
+        # TODO: Don't add local imports to the global namespace
+
         # Miscompiling imports inside of functions, like below in import from.
         if as_name:
             name = as_name
@@ -124,12 +151,10 @@ def transform_import(builder: IRBuilder, node: Import) -> None:
         else:
             base = name = node_id.split('.')[0]
 
-        # Python 3.7 has a nice 'PyImport_GetModule' function that we can't use :(
-        mod_dict = builder.primitive_op(get_module_dict_op, [], node.line)
-        obj = builder.primitive_op(dict_get_item_op,
-                                [mod_dict, builder.load_static_unicode(base)], node.line)
+        obj = builder.get_module(base, node.line)
+
         builder.gen_method_call(
-            globals, '__setitem__', [builder.load_static_unicode(name), obj],
+            globals, '__setitem__', [builder.load_str(name), obj],
             result_type=None, line=node.line)
 
 
@@ -140,30 +165,29 @@ def transform_import_from(builder: IRBuilder, node: ImportFrom) -> None:
     module_state = builder.graph[builder.module_name]
     if module_state.ancestors is not None and module_state.ancestors:
         module_package = module_state.ancestors[0]
+    elif builder.module_path.endswith("__init__.py"):
+        module_package = builder.module_name
     else:
         module_package = ''
 
     id = importlib.util.resolve_name('.' * node.relative + node.id, module_package)
 
-    builder.gen_import(id, node.line)
-    module = builder.load_module(id)
+    globals = builder.load_globals_dict()
+    imported_names = [name for name, _ in node.names]
+    module = builder.gen_import_from(id, globals, imported_names, node.line)
 
     # Copy everything into our module's dict.
     # Note that we miscompile import from inside of functions here,
     # since that case *shouldn't* load it into the globals dict.
     # This probably doesn't matter much and the code runs basically right.
-    globals = builder.load_globals_dict()
     for name, maybe_as_name in node.names:
-        # If one of the things we are importing is a module,
-        # import it as a module also.
-        fullname = id + '.' + name
-        if fullname in builder.graph or fullname in module_state.suppressed:
-            builder.gen_import(fullname, node.line)
-
         as_name = maybe_as_name or name
-        obj = builder.py_get_attr(module, name, node.line)
+        obj = builder.call_c(import_from_op,
+                             [module, builder.load_str(id),
+                              builder.load_str(name), builder.load_str(as_name)],
+                             node.line)
         builder.gen_method_call(
-            globals, '__setitem__', [builder.load_static_unicode(as_name), obj],
+            globals, '__setitem__', [builder.load_str(as_name), obj],
             result_type=None, line=node.line)
 
 
@@ -217,6 +241,9 @@ def transform_while_stmt(builder: IRBuilder, s: WhileStmt) -> None:
 
 
 def transform_for_stmt(builder: IRBuilder, s: ForStmt) -> None:
+    if s.is_async:
+        builder.error('async for is unimplemented', s.line)
+
     def body() -> None:
         builder.accept(s.body)
 
@@ -238,12 +265,12 @@ def transform_continue_stmt(builder: IRBuilder, node: ContinueStmt) -> None:
 
 def transform_raise_stmt(builder: IRBuilder, s: RaiseStmt) -> None:
     if s.expr is None:
-        builder.primitive_op(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
+        builder.call_c(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
         builder.add(Unreachable())
         return
 
     exc = builder.accept(s.expr)
-    builder.primitive_op(raise_exception_op, [exc], s.line)
+    builder.call_c(raise_exception_op, [exc], s.line)
     builder.add(Unreachable())
 
 
@@ -278,7 +305,7 @@ def transform_try_except(builder: IRBuilder,
     # exception is raised, based on the exception in exc_info.
     builder.builder.push_error_handler(double_except_block)
     builder.activate_block(except_entry)
-    old_exc = builder.maybe_spill(builder.primitive_op(error_catch_op, [], line))
+    old_exc = builder.maybe_spill(builder.call_c(error_catch_op, [], line))
     # Compile the except blocks with the nonlocal control flow overridden to clear exc_info
     builder.nonlocal_control.append(
         ExceptNonlocalControl(builder.nonlocal_control[-1], old_exc))
@@ -288,16 +315,16 @@ def transform_try_except(builder: IRBuilder,
         next_block = None
         if type:
             next_block, body_block = BasicBlock(), BasicBlock()
-            matches = builder.primitive_op(
+            matches = builder.call_c(
                 exc_matches_op, [builder.accept(type)], type.line
             )
-            builder.add(Branch(matches, body_block, next_block, Branch.BOOL_EXPR))
+            builder.add(Branch(matches, body_block, next_block, Branch.BOOL))
             builder.activate_block(body_block)
         if var:
             target = builder.get_assignment_target(var)
             builder.assign(
                 target,
-                builder.primitive_op(get_exc_value_op, [], var.line),
+                builder.call_c(get_exc_value_op, [], var.line),
                 var.line
             )
         handler_body()
@@ -307,7 +334,7 @@ def transform_try_except(builder: IRBuilder,
 
     # Reraise the exception if needed
     if next_block:
-        builder.primitive_op(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
+        builder.call_c(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
         builder.add(Unreachable())
 
     builder.nonlocal_control.pop()
@@ -317,15 +344,15 @@ def transform_try_except(builder: IRBuilder,
     # restore the saved exc_info information and continue propagating
     # the exception if it exists.
     builder.activate_block(cleanup_block)
-    builder.primitive_op(restore_exc_info_op, [builder.read(old_exc)], line)
+    builder.call_c(restore_exc_info_op, [builder.read(old_exc)], line)
     builder.goto(exit_block)
 
     # Cleanup for if we leave except through a raised exception:
     # restore the saved exc_info information and continue propagating
     # the exception.
     builder.activate_block(double_except_block)
-    builder.primitive_op(restore_exc_info_op, [builder.read(old_exc)], line)
-    builder.primitive_op(keep_propagating_op, [], NO_TRACEBACK_LINE_NO)
+    builder.call_c(restore_exc_info_op, [builder.read(old_exc)], line)
+    builder.call_c(keep_propagating_op, [], NO_TRACEBACK_LINE_NO)
     builder.add(Unreachable())
 
     # If present, compile the else body in the obvious way
@@ -376,7 +403,7 @@ def try_finally_entry_blocks(builder: IRBuilder,
                              main_entry: BasicBlock,
                              finally_block: BasicBlock,
                              ret_reg: Optional[Register]) -> Value:
-    old_exc = builder.alloc_temp(exc_rtuple)
+    old_exc = Register(exc_rtuple)
 
     # Entry block for non-exceptional flow
     builder.activate_block(main_entry)
@@ -402,7 +429,7 @@ def try_finally_entry_blocks(builder: IRBuilder,
                 builder.add(LoadErrorValue(builder.ret_types[-1]))
             )
         )
-    builder.add(Assign(old_exc, builder.primitive_op(error_catch_op, [], -1)))
+    builder.add(Assign(old_exc, builder.call_c(error_catch_op, [], -1)))
     builder.goto(finally_block)
 
     return old_exc
@@ -442,7 +469,7 @@ def try_finally_resolve_control(builder: IRBuilder,
 
     # Reraise the exception if there was one
     builder.activate_block(reraise)
-    builder.primitive_op(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
+    builder.call_c(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
     builder.add(Unreachable())
     builder.builder.pop_error_handler()
 
@@ -463,7 +490,7 @@ def try_finally_resolve_control(builder: IRBuilder,
     # If there was an exception, restore again
     builder.activate_block(cleanup_block)
     finally_control.gen_cleanup(builder, -1)
-    builder.primitive_op(keep_propagating_op, [], NO_TRACEBACK_LINE_NO)
+    builder.call_c(keep_propagating_op, [], NO_TRACEBACK_LINE_NO)
     builder.add(Unreachable())
 
     return out_block
@@ -520,7 +547,7 @@ def transform_try_stmt(builder: IRBuilder, t: TryStmt) -> None:
 
 
 def get_sys_exc_info(builder: IRBuilder) -> List[Value]:
-    exc_info = builder.primitive_op(get_exc_info_op, [], -1)
+    exc_info = builder.call_c(get_exc_info_op, [], -1)
     return [builder.add(TupleGet(exc_info, i, -1)) for i in range(3)]
 
 
@@ -534,13 +561,13 @@ def transform_with(builder: IRBuilder,
     # We could probably optimize the case where the manager is compiled by us,
     # but that is not our common case at all, so.
     mgr_v = builder.accept(expr)
-    typ = builder.primitive_op(type_op, [mgr_v], line)
+    typ = builder.call_c(type_op, [mgr_v], line)
     exit_ = builder.maybe_spill(builder.py_get_attr(typ, '__exit__', line))
     value = builder.py_call(
         builder.py_get_attr(typ, '__enter__', line), [mgr_v], line
     )
     mgr = builder.maybe_spill(mgr_v)
-    exc = builder.maybe_spill_assignable(builder.primitive_op(true_op, [], -1))
+    exc = builder.maybe_spill_assignable(builder.true())
 
     def try_body() -> None:
         if target:
@@ -548,7 +575,7 @@ def transform_with(builder: IRBuilder,
         body()
 
     def except_body() -> None:
-        builder.assign(exc, builder.primitive_op(false_op, [], -1), line)
+        builder.assign(exc, builder.false(), line)
         out_block, reraise_block = BasicBlock(), BasicBlock()
         builder.add_bool_branch(
             builder.py_call(builder.read(exit_),
@@ -557,14 +584,14 @@ def transform_with(builder: IRBuilder,
             reraise_block
         )
         builder.activate_block(reraise_block)
-        builder.primitive_op(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
+        builder.call_c(reraise_exception_op, [], NO_TRACEBACK_LINE_NO)
         builder.add(Unreachable())
         builder.activate_block(out_block)
 
     def finally_body() -> None:
         out_block, exit_block = BasicBlock(), BasicBlock()
         builder.add(
-            Branch(builder.read(exc), exit_block, out_block, Branch.BOOL_EXPR)
+            Branch(builder.read(exc), exit_block, out_block, Branch.BOOL)
         )
         builder.activate_block(exit_block)
         none = builder.none_object()
@@ -585,6 +612,9 @@ def transform_with(builder: IRBuilder,
 
 
 def transform_with_stmt(builder: IRBuilder, o: WithStmt) -> None:
+    if o.is_async:
+        builder.error('async with is unimplemented', o.line)
+
     # Generate separate logic for each expr in it, left to right
     def generate(i: int) -> None:
         if i >= len(o.expr):
@@ -614,7 +644,7 @@ def transform_assert_stmt(builder: IRBuilder, a: AssertStmt) -> None:
         message = builder.accept(a.msg)
         exc_type = builder.load_module_attr_by_fullname('builtins.AssertionError', a.line)
         exc = builder.py_call(exc_type, [message], a.line)
-        builder.primitive_op(raise_exception_op, [exc], a.line)
+        builder.call_c(raise_exception_op, [exc], a.line)
     builder.add(Unreachable())
     builder.activate_block(ok_block)
 
@@ -633,8 +663,15 @@ def transform_del_item(builder: IRBuilder, target: AssignmentTarget, line: int)
             line=line
         )
     elif isinstance(target, AssignmentTargetAttr):
-        key = builder.load_static_unicode(target.attr)
-        builder.add(PrimitiveOp([target.obj, key], py_delattr_op, line))
+        if isinstance(target.obj_type, RInstance):
+            cl = target.obj_type.class_ir
+            if not cl.is_deletable(target.attr):
+                builder.error('"{}" cannot be deleted'.format(target.attr), line)
+                builder.note(
+                    'Using "__deletable__ = ' +
+                    '[\'<attr>\']" in the class body enables "del obj.<attr>"', line)
+        key = builder.load_str(target.attr)
+        builder.call_c(py_delattr_op, [target.obj, key], line)
     elif isinstance(target, AssignmentTargetRegister):
         # Delete a local by assigning an error value to it, which will
         # prompt the insertion of uninit checks.
