@@ -14,7 +14,7 @@ coercion is necessary first.
 """
 
 from mypyc.ir.rtypes import (
-    RType, RUnion, RInstance, RPrimitive, RTuple, RVoid, RTypeVisitor, RStruct,
+    RType, RUnion, RInstance, RPrimitive, RTuple, RVoid, RTypeVisitor, RStruct, RArray,
     is_int_rprimitive, is_short_int_rprimitive, is_bool_rprimitive, is_bit_rprimitive
 )
 from mypyc.subtype import is_subtype
@@ -56,5 +56,8 @@ class RTSubtypeVisitor(RTypeVisitor[bool]):
     def visit_rstruct(self, left: RStruct) -> bool:
         return isinstance(self.right, RStruct) and self.right.name == left.name
 
+    def visit_rarray(self, left: RArray) -> bool:
+        return left == self.right
+
     def visit_rvoid(self, left: RVoid) -> bool:
         return isinstance(self.right, RVoid)
