@@ -1,7 +1,10 @@
 # Simple support library for our run tests.
 
 from contextlib import contextmanager
-from typing import Iterator, TypeVar, Generator, Optional, List, Tuple, Sequence, Union
+from typing import (
+    Any, Iterator, TypeVar, Generator, Optional, List, Tuple, Sequence,
+    Union, Callable,
+)
 
 @contextmanager
 def assertRaises(typ: type, msg: str = '') -> Iterator[None]:
@@ -20,7 +23,7 @@ V = TypeVar('V')
 def run_generator(gen: Generator[T, V, U],
                   inputs: Optional[List[V]] = None,
                   p: bool = False) -> Tuple[Sequence[T], Union[U, str]]:
-    res = []  # type: List[T]
+    res: List[T] = []
     i = -1
     while True:
         try:
@@ -37,3 +40,13 @@ def run_generator(gen: Generator[T, V, U],
             print(val)
         res.append(val)
         i += 1
+
+F = TypeVar('F', bound=Callable)
+
+
+# Wrap a mypyc-generated function in a real python function, to allow it to be
+# stuck into classes and the like.
+def make_python_function(f: F) -> F:
+    def g(*args: Any, **kwargs: Any) -> Any:
+        return f(*args, **kwargs)
+    return g  # type: ignore
