@@ -16,11 +16,13 @@ from mypyc.transform.exceptions import insert_exception_handling
 from mypyc.transform.refcount import insert_ref_count_opcodes
 from mypyc.test.testutil import (
     ICODE_GEN_BUILTINS, use_custom_builtins, MypycDataSuite, build_ir_for_single_file,
-    assert_test_output, remove_comment_lines, replace_native_int
+    assert_test_output, remove_comment_lines
 )
+from mypyc.analysis.blockfreq import frequently_executed_blocks
 
 files = [
-    'exceptions.test'
+    'exceptions.test',
+    'exceptions-freq.test',
 ]
 
 
@@ -32,7 +34,6 @@ class TestExceptionTransform(MypycDataSuite):
         """Perform a runtime checking transformation test case."""
         with use_custom_builtins(os.path.join(self.data_prefix, ICODE_GEN_BUILTINS), testcase):
             expected_output = remove_comment_lines(testcase.output)
-            expected_output = replace_native_int(expected_output)
             try:
                 ir = build_ir_for_single_file(testcase.input)
             except CompileError as e:
@@ -47,6 +48,9 @@ class TestExceptionTransform(MypycDataSuite):
                     insert_exception_handling(fn)
                     insert_ref_count_opcodes(fn)
                     actual.extend(format_func(fn))
+                    if testcase.name.endswith('_freq'):
+                        common = frequently_executed_blocks(fn.blocks[0])
+                        actual.append('hot blocks: %s' % sorted(b.label for b in common))
 
             assert_test_output(testcase, actual, 'Invalid source code output',
                                expected_output)
