@@ -9,7 +9,7 @@ setup(
     author="The mypy team",
     version='0.1',
     package_data={'typedpkg': ['py.typed']},
-    packages=['typedpkg'],
+    packages=['typedpkg', 'typedpkg.pkg'],
     include_package_data=True,
     zip_safe=False,
 )
